REM REVISION HISTORY

REM 1.1 - added lock/unlock EEPROM support
REM       uses high speed eeprom write (&hfa)
REM 2.1 - corrected temperature calibration formulas in ReadBattery Data
REM       and CalibrateTemperature functions
REM       added support for negative temperatures
REM       altered method for ensuring the user switched v,i,r
REM       changed data format units in top-of-page form


DEFINT A-Z

DECLARE FUNCTION BroadCastOff ()
DECLARE FUNCTION CalibrateCurrent ()
DECLARE FUNCTION CalibrateTemperature ()
DECLARE FUNCTION CalibrateVoltage ()
DECLARE FUNCTION CommCheck ()
DECLARE FUNCTION DnLoadOrig ()
DECLARE FUNCTION DownLoad ()
DECLARE FUNCTION EndTest ()
DECLARE FUNCTION EscCont ()
DECLARE FUNCTION Initialize ()
DECLARE FUNCTION InitTG ()
DECLARE FUNCTION loadee ()
DECLARE FUNCTION More ()
DECLARE FUNCTION ReadBatteryData ()
DECLARE FUNCTION ReadEE (location AS INTEGER)
DECLARE FUNCTION ReadRam (location AS INTEGER)
DECLARE FUNCTION ResetTG ()
DECLARE FUNCTION Response (msg AS STRING, Valid AS STRING)
DECLARE FUNCTION RxData ()
DECLARE FUNCTION SetNormalization ()
DECLARE FUNCTION Test ()
DECLARE FUNCTION WriteEE (location AS INTEGER, Value AS INTEGER)
DECLARE FUNCTION YesNo ()

DECLARE SUB CalibLog (cmd AS INTEGER, DetailLine AS STRING)
DECLARE SUB clrLine25 ()
DECLARE SUB DnLoadEEPROM ()
DECLARE SUB dspCommPort ()
DECLARE SUB dspError (msg AS STRING)
DECLARE SUB dspFactors ()
DECLARE SUB dspRevision ()
DECLARE SUB ePRINT (msg AS STRING)
DECLARE SUB FlushRcvr ()
DECLARE SUB Form ()
DECLARE SUB GetEEPROMfile ()
DECLARE SUB GetLogFile ()
DECLARE SUB GetPartNumber ()
DECLARE SUB GetPortNumber ()
DECLARE SUB GetRefCur ()
DECLARE SUB Greetings ()
DECLARE SUB InitEE ()
DECLARE SUB PrintReport ()
DECLARE SUB prtFactors ()
DECLARE SUB setVid (dType AS INTEGER)
DECLARE SUB UnlockEEPROM ()
DECLARE SUB UpLoadOrig ()

DIM SHARED PortNumber AS INTEGER
DIM SHARED EEdata(128) AS INTEGER
DIM SHARED EEdataAvail AS INTEGER
DIM SHARED MenuIndex  AS INTEGER
DIM SHARED OrigParams(0 TO 9) AS INTEGER

DIM SHARED Filename AS STRING
DIM SHARED PartNumber AS STRING
DIM SHARED SysErr AS INTEGER
DIM SHARED Revision AS INTEGER

' Battery Data Parameters
DIM SHARED Voltage AS LONG
DIM SHARED Temperature AS LONG
DIM SHARED Current AS LONG
DIM SHARED Capacity AS LONG
DIM SHARED PctChg AS INTEGER
DIM SHARED ErrorByte AS INTEGER
DIM SHARED FlagByte AS INTEGER

DIM SHARED hLogFile AS INTEGER
DIM SHARED hEEPROM AS INTEGER
DIM SHARED RefCur AS INTEGER

CONST dPROMPT = 1       ' Display formatting info
CONST dERROR = 2
CONST dVERSION = 3
CONST dSAVE = 4
CONST dRESTORE = 5
CONST dBATTERY = 6
CONST dCURSLOPE = 7
CONST dTEMPSLOPE = 8
CONST dTEMPOFF = 9
CONST dVOLTSLOPE = 10
CONST dVOLTOFF = 11
CONST dCURSLOPEORIG = 12
CONST dTEMPSLOPEORIG = 13
CONST dTEMPOFFORIG = 14
CONST dVOLTSLOPEORIG = 15
CONST dVOLTOFFORIG = 16
CONST dMENUBAR = 17
CONST dCOMMPORT = 18
CONST dEPRINT = 19

CONST NBRMENUITEMS = 9
CONST SIGNON = "MTA11200    TrueGauge Calibration Program    (Ver: 2.1)"
CONST REVISION24 = &H24

    DIM rc AS INTEGER   ' rc = -1 => skip to previous procedure (TAB)
			' rc =  0 => continue
			' rc =  1 => skip to next (BACKTAB)

    ON ERROR GOTO CommErr

    EEdataAvail = 0
    hLogFile = 0
    MenuIndex = 0
    RefCur = 1000

    Form

    DO
	setVid (dMENUBAR)

	SELECT CASE MenuIndex

	    CASE 0
		rc = Initialize

	    CASE 1
		rc = Test

	    CASE 2
		rc = SetNormalization
'               IF rc = 0 THEN rc = ResetTG

	    CASE 3
		rc = CalibrateVoltage

	    CASE 4
		rc = CalibrateCurrent

	    CASE 5
		rc = CalibrateTemperature

	    CASE 6
		rc = EndTest

	    CASE 7
		rc = DnLoadOrig

	    CASE 8
		rc = More
		IF rc = 0 THEN EXIT DO

	END SELECT

	REM: For normal processing (Enter key) loop back to the Load EEPROM
	REM: procedure at the end of Calibrate Voltage, else go to next
	REM: procedure.
	REM:
	REM: If tab or backtab was hit move to next/previous
	REM:
	REM: If a port has not been opened successfully, then skip start,
	REM: set NP, calibratations, and restore procedures
	REM:

	IF rc = 0 THEN  ' Normal operation--go back to start of calibration loop
	    IF MenuIndex = 6 THEN MenuIndex = 0
	    rc = 1
	END IF
	
	DO
	    MenuIndex = MenuIndex + rc
	    IF MenuIndex = NBRMENUITEMS THEN MenuIndex = 0
	    IF MenuIndex = -1 THEN MenuIndex = NBRMENUITEMS - 1
	    IF MenuIndex < 1 OR MenuIndex > 7 THEN EXIT DO
	    IF PortNumber <> 0 THEN EXIT DO
	LOOP

    LOOP

    IF PortNumber > 0 THEN
	CLOSE #1
    END IF

    IF hLogFile <> 0 THEN
	CalibLog 1, "calibration program terminated"
	CLOSE #hLogFile
    END IF

    END


CommErr:
	DIM errMsg AS STRING

	SysErr = ERR

	SELECT CASE ERR
		CASE 24
			errMsg = "No response on Com Port" + STR$(PortNumber)
			PortNumber = 0
			CLOSE #1
			Form

		CASE 6
			errMsg = "Arithmetic Overflow"

		CASE 11
			errMsg = "Division by zero"

		CASE 52
			errMsg = "Bad File Name"

		CASE 64
			errMsg = "Port Not Available"

		CASE 53
			errMsg = ""
		CASE ELSE
			errMsg = "Unknown Error: " + STR$(ERR)

	END SELECT

	IF errMsg <> "" THEN dspError errMsg

	RESUME NEXT

FUNCTION BroadCastOff

REM: return 0 => continue
REM:        1 => broadcast enabled, do not continue

    DIM BeginTime AS INTEGER

    PRINT "Checking for Broadcast Enabled. ";

    temp = RxData
    IF temp <> -1 THEN
	PRINT "Broadcast mode is enabled."
	CalibLog 1, "broadcast mode is enabled."
	PRINT "Do you want to disable and continue (y or n)?";
	IF YesNo = 0 THEN
	    BroadCastOff = 1
	    CLOSE #1
	    PortNumber = 0
	    EXIT FUNCTION
	END IF

	REM: Disable broadcast mode
	PRINT "Disabling broadcast.  One moment, please."
	CalibLog 1, "disabled broadcast mode."
	temp = WriteEE(43, 0)

	FlushRcvr       ' flush rx buffer
    ELSE
	PRINT "Broadcasting is disabled."
    END IF

    BroadCastOff = 0

END FUNCTION

SUB CalibLog (cmd AS INTEGER, DetailLine AS STRING)

    DIM slen AS INTEGER

    IF hLogFile <> 0 THEN
	SELECT CASE cmd
	    CASE 0                       ' print seperator line
		PRINT #hLogFile, STRING$(78, 178)

	    CASE 1                      ' print detail line
		PRINT #hLogFile, DATE$ + " " + TIME$ + "   " + DetailLine

	    CASE 2                      ' print calib signon
		PRINT #hLogFile,
		slen = (80 - LEN(SIGNON)) / 2
		PRINT #hLogFile, STRING$(slen, 178) + " " + SIGNON + " " + STRING$(slen, 178)

	END SELECT
    END IF

END SUB

FUNCTION CalibrateCurrent

    DIM i AS INTEGER
    DIM i1 AS LONG
    DIM i2 AS LONG
    DIM m AS LONG
    DIM isc(2) AS INTEGER
    DIM tmp AS INTEGER

    setVid (dPROMPT)
    ePRINT "Calibrate Current."
    rc = Response("<Enter=Calibrate>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")

    isc(0) = 0
    isc(1) = 1

    IF rc = 0 THEN
	DO
	    CalibLog 1, "Calibrate Current"
	    PRINT "Initializing correction factors.  One moment, please: []";
	    LOCATE , POS(0) - 3

	    FOR i = 0 TO 1
		IF PortNumber = 0 THEN EXIT FUNCTION
		IF WriteEE(54 + i, isc(i)) <> isc(i) THEN
		    dspError "Error in initializing current correction."
		    EXIT DO
		ELSE
		    PRINT "*";
		END IF
	    NEXT i
	       
	    PRINT "Set current to 0 mA then hit any key (Press Esc to Quit)."
	    tmp = EscCont

	    IF ReadBatteryData = 0 THEN
		dspError "Error in reading current"
		EXIT DO
	    END IF

	    i1 = Current

	    PRINT "Set current to " + STR$(-RefCur) + "mA then hit any key."
	    tmp = EscCont

	    IF ReadBatteryData = 0 THEN
		dspError "Error in reading current"
		EXIT DO
	    END IF

	    i2 = Current

	    REM: Calculate slope

	    IF i2 <> i1 THEN 'This ensures that the user
			     'switched currents

		m = INT((RefCur / (i2 - i1)) * 256)
		isc(1) = INT(m / 256)
		isc(0) = m AND &HFF

		setVid (dSAVE)
		setVid (dCURSLOPE)
		PRINT USING "#.###"; m / 256
		setVid (dRESTORE)
		CalibLog 1, "calibrated current slope:" + STR$(m / 256)

		UnlockEEPROM

		PRINT "Storing new correction factors.  One moment, please: []";
		LOCATE , POS(0) - 3

		FOR i = 0 TO 1
		    IF PortNumber = 0 THEN EXIT FUNCTION
		    IF WriteEE(54 + i, isc(i)) <> isc(i) THEN
			dspError "Error in initializing current correction."
			EXIT DO
		    ELSE
			PRINT "*";
		    END IF
		NEXT i

		PRINT
		PrintReport

		EXIT DO
	    ELSE        ' current setting probably not changed
		dspError "Received the same readings for both settings."
		setVid (dPROMPT)
		PRINT "Do you want to repeat calibration procedure (y or n)?";
		IF YesNo = 0 THEN
		    setVid (dSAVE)
		    setVid (dCURSLOPE)
		    PRINT USING "#.###"; 1
		    setVid (dRESTORE)
		    EXIT DO
		END IF
	    END IF

	LOOP

    END IF

    CalibrateCurrent = rc

END FUNCTION

FUNCTION CalibrateTemperature

    DIM tsc(4) AS INTEGER
    DIM m AS LONG
    DIM b AS LONG
    DIM b2 AS LONG
    DIM t40 AS LONG
    DIM t10 AS LONG
    DIM tmp AS INTEGER

    setVid (dPROMPT)
    ePRINT "Calibrate Temperature."
    rc = Response("<Enter=Calibrate>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")

    IF rc = 0 THEN
	CalibLog 1, "Calibrate Temperature"
	tsc(0) = 0      ' initial calibration values
	tsc(1) = 1      ' set gain (slope) to 1
	tsc(2) = 0
	tsc(3) = 0      ' set offset (y intercept) to 0

	DO
	    PRINT "Initializing correction factors.  One moment, please: []";
	    LOCATE , POS(0) - 5

	    FOR i = 0 TO 3
		PRINT "*";
		IF PortNumber = 0 THEN EXIT FUNCTION
		IF WriteEE(56 + i, tsc(i)) <> tsc(i) THEN
		    dspError "Error in storing temperature correction."
		    EXIT DO
		END IF
	    NEXT

	    PRINT "Set thermistor to 51058 Ohms then hit any key."

	    tmp = EscCont

	    IF ReadBatteryData = 0 THEN
		dspError "Error in reading temperature."
		EXIT DO
	    END IF

	    t40 = Temperature       ' point B: (40, t40)

	    IF t40 > 128 THEN ' This loop, when appropriate,
	       t40 = t40 - 256' changes the temperature to a
	    END IF            ' proper negative reading.

	    PRINT "Set thermistor to 207807 Ohms then hit any key."
	    tmp = EscCont

	    IF ReadBatteryData = 0 THEN
		dspError "Error in reading temperature."
		EXIT DO
	    END IF

	    t10 = Temperature       ' point A: (10, t10)

	    IF t10 > 128 THEN ' This loop, when appropriate,
	       t10 = t10 - 256' changes the temperature to a
	    END IF            ' proper negative reading.

	    REM: Calculate slope  m = (y2 - y1)/(x2 - x1)

	    IF t40 <> t10 THEN 'This ensures that the user
			       'switched resistors

		m = INT(((10! - 40!) / (t10 - t40)) * 256)
		tsc(0) = INT(m AND &HFF)
		tsc(1) = INT(m / &H100)

	    REM: Calculate y intercept b using (y = mx + b), then (b = y - mx)

		b = INT((10 - ((m / 256) * t10)) * 256)

		IF b > 0 THEN
		    b2 = b
		    ELSE
		    IF b = &H8000 THEN
			b2 = &H8000
			ELSE
			    b2 = b + &H10000
		    END IF
		END IF

		tsc(2) = INT(b2 AND &HFF)        ' lsb
		tsc(3) = INT(b2 / &H100)         ' msb

		setVid (dSAVE)
		setVid (dTEMPSLOPE)
		PRINT USING "#.###"; m / 256
		setVid (dTEMPOFF)
		PRINT USING "#####"; b2
		setVid (dRESTORE)

		CalibLog 1, "calibrated temperature slope:" + STR$(m / 256)
		CalibLog 1, "calibrated temperature offset:" + STR$(b2)

		UnlockEEPROM

		PRINT "Storing new correction factors.  One moment, please: []";
		LOCATE , POS(0) - 5

		FOR i = 0 TO 3
		    PRINT "*";
		    IF PortNumber = 0 THEN EXIT FUNCTION
		    IF WriteEE(56 + i, tsc(i)) <> tsc(i) THEN
			dspError "Error in storing temperature correction."
			EXIT DO
		    END IF
		NEXT

		PRINT
		PrintReport

		EXIT DO
	    ELSE        ' current setting probably not changed
		dspError "Received the same readings for both settings."
		setVid (dPROMPT)
		PRINT "Do you want to repeat calibration procedure (y or n)?";
		IF YesNo = 0 THEN
		    setVid (dSAVE)
		    setVid (dTEMPSLOPE)
		    PRINT USING "#.###"; 1
		    setVid (dTEMPOFF)
		    PRINT USING "#####"; 0
		    setVid (dRESTORE)
		    EXIT DO
		END IF
	    END IF

	LOOP

    END IF

    CalibrateTemperature = rc

END FUNCTION

FUNCTION CalibrateVoltage

    DIM i AS INTEGER
    DIM v1 AS LONG
    DIM v2 AS LONG
    DIM m AS LONG
    DIM b AS LONG

    DIM vsc(3) AS INTEGER
    DIM tmp AS INTEGER

    setVid (dPROMPT)
    ePRINT "Calibrate Voltage."
    rc = Response("<Enter=Calibrate>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")

    IF rc = 0 THEN
	CalibLog 1, "Calibrate Voltage"
	vsc(0) = 0      ' initial calibration values
	vsc(1) = 1
	vsc(2) = 0
	vsc(3) = 0

	DO
	    PRINT "Initializing correction factors.  One moment, please: []";
	    LOCATE , POS(0) - 5
       
	    FOR i = 0 TO 3
		PRINT "*";
		IF PortNumber = 0 THEN EXIT FUNCTION
		IF WriteEE(50 + i, vsc(i)) <> vsc(i) THEN
		    dspError "Error in initializing voltage correction."
		    EXIT DO
		END IF
	    NEXT

	    PRINT

	    PRINT "Set voltage to 6.000 Volts then hit any key."
	    tmp = EscCont
	    IF ReadBatteryData = 0 THEN
		dspError "Error in reading voltage."
		EXIT DO
	    END IF

	    v1 = Voltage

	    PRINT "Set voltage to 14.000 Volts then hit any key."
	    tmp = EscCont
	    IF ReadBatteryData = 0 THEN
		dspError "Error in reading voltage."
		EXIT DO
	    END IF

	    v2 = Voltage

	    REM:    Calculate slope and intercept

	    IF v2 <> v1 THEN 'This ensures that the user
			     'switched voltages

		m = INT(((14000 - 6000) / (v2 - v1)) * 256)
		vsc(1) = INT(m / 256)
		vsc(0) = m AND &HFF
		b = 14000 - (m * v2) / 256
		IF b < 0 THEN b = b + 65536
		vsc(3) = INT(b / 256)
		vsc(2) = b AND &HFF

		setVid (dSAVE)
		setVid (dVOLTSLOPE)
		PRINT USING "#.###"; m / 256
		setVid (dVOLTOFF)
		PRINT USING "#####"; b
		setVid (dRESTORE)

		CalibLog 1, "calibrated voltage slope:" + STR$(m / 256)
		CalibLog 1, "calibrated voltage offset:" + STR$(b)

		UnlockEEPROM

		PRINT "Storing new correction factors.  One moment, please: []";
		LOCATE , POS(0) - 5

		FOR i = 0 TO 3
		    PRINT "*";
		    IF PortNumber = 0 THEN EXIT FUNCTION
		    IF WriteEE(50 + i, vsc(i)) <> vsc(i) THEN
			dspError "Error in storing voltage correction."
			EXIT DO
		    END IF
		NEXT

		PRINT
		PrintReport

		EXIT DO

	    ELSE        ' voltage setting probably not changed
		dspError "Received the same readings for both settings."
		setVid (dPROMPT)
		PRINT "Do you want to repeat calibration procedure (y or n)?";
		IF YesNo = 0 THEN
		    setVid (dSAVE)
		    setVid (dVOLTSLOPE)
		    PRINT USING "#.###"; 1
		    setVid (dVOLTOFF)
		    PRINT USING "#####"; 0
		    setVid (dRESTORE)
		    EXIT DO
		END IF

	    END IF

	 LOOP

     END IF

     CalibrateVoltage = rc

END FUNCTION

SUB clrLine25

    setVid (dSAVE)          ' clear error message line
    LOCATE 25, 1
    PRINT STRING$(80, 32);
    setVid (dRESTORE)

END SUB

FUNCTION CommCheck%

REM check communication link with truegauge

    FlushRcvr
    PRINT #1, CHR$(&HF3)
    Revision = RxData

    IF Revision <> -1 THEN
	dspCommPort
	dspRevision
	UnlockEEPROM
	CommCheck = 0
	IF ReadBatteryData = 1 THEN
	    PrintReport
	    PRINT
	END IF
	UpLoadOrig

    ELSE
	CLOSE #1
	PortNumber = 0
	CommCheck = 1
    END IF

END FUNCTION

SUB DnLoadEEPROM

    SHARED Filename AS STRING
    SHARED EEdata() AS INTEGER

    setVid (dPROMPT)
   
    PRINT "DownLoad data into TrueGauge EEPROM from file " + Filename
    rc = Response("<Enter=DownLoad>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")

    IF rc = 0 THEN

	REM: Write to EEPROM and verify (D-->A)

	PRINT "Download to EEPROM. This will OVERWRITE the current contents."
	PRINT "Do you want to continue (y or n)?";
	IF YesNo = 0 THEN EXIT SUB

	PRINT "Begin Download"
	PRINT "Press any key to TERMINATE download"
	PRINT

	CalibLog 1, "initializing (downloading) EEPROM data."
	PRINT #1, CHR$(&HFA);

	FOR i = 0 TO 127
	    IF INKEY$ <> "" THEN
		CalibLog 1, "download aborted."
		EXIT SUB
	    END IF

	    LOCATE CSRLIN, 1    ' stay on same line
	    PRINT "...writing " + HEX$(EEdata(i)) + " into location " + HEX$(i);
	    PRINT #1, CHR$(EEdata(i));
	    IF RxData <> EEdata(i) THEN
		PRINT "??? EEPROM data download failed at location " + HEX$(i) + ", value is " + HEX$(EEdata(i))
		EXIT FOR
	    END IF
	NEXT

	CalibLog 1, "download complete."

    END IF

END SUB

FUNCTION DnLoadOrig

    DIM i AS INTEGER
    DIM byte AS INTEGER
				 
    setVid (dPROMPT)
  
    ePRINT "Restore Original Calibration Values"
    rc = Response("<Enter=Restore>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")

    IF rc = 0 THEN
	Form
	dspRevision
	dspCommPort
	PRINT "Restoring Original Calibration Values: []";
	CalibLog 1, "restored original calibration values."
	LOCATE , POS(0) - 11

	FOR i = 0 TO 9
	    IF PortNumber = 0 THEN EXIT FUNCTION
	    IF WriteEE(50 + i, OrigParams(i)) <> OrigParams(i) THEN
		dspError "Error writing EEPROM location" + STR$(50 + i)
		EXIT FUNCTION
	    END IF

	    PRINT "*";

	NEXT

	PRINT

	dspFactors
	PrintReport
	PRINT

    END IF

    DnLoadOrig = rc

END FUNCTION

SUB dspCommPort

    setVid (dSAVE)
    setVid (dCOMMPORT)
    PRINT " Communication Link on Comm Port" + STR$(PortNumber) + " ";
    setVid (dRESTORE)

END SUB

SUB dspError (msg AS STRING)

	DIM lMsg AS INTEGER
	DIM errMsg AS STRING
	DIM xMsg AS STRING

	setVid (dSAVE)

	xMsg = msg + "    <Enter=Continue>"
	lMsg = LEN(xMsg)
	setVid (dERROR)
	errMsg = STRING$(80, 32)
	MID$(errMsg, 40 - lMsg / 2) = xMsg
	PRINT errMsg;

	WHILE (Response("", "") <> 0)
	WEND

	setVid (dRESTORE)

	clrLine25

	CalibLog 1, "ERROR: " + msg

END SUB

SUB dspFactors

    setVid (dSAVE)

    setVid (dVOLTSLOPEORIG)
    PRINT USING "##.###"; (OrigParams(1) * 256 + OrigParams(0)) / 256!
    setVid (dVOLTOFFORIG)
    PRINT USING "#####"; OrigParams(3) * 256 + OrigParams(2)

    setVid (dTEMPSLOPEORIG)
    PRINT USING "##.###"; (OrigParams(7) * 256 + OrigParams(6)) / 256!
    setVid (dTEMPOFFORIG)
    PRINT OrigParams(9) * 256 + OrigParams(8)

    setVid (dCURSLOPEORIG)
    PRINT USING "##.###"; (OrigParams(5) * 256 + OrigParams(4)) / 256!

    setVid (dRESTORE)

END SUB

SUB dspRevision

    setVid (dSAVE)
    setVid (dVERSION)
    PRINT " Firmware Level:" + STR$(INT(Revision / 16)) + "." + LTRIM$(STR$(INT(Revision AND &HF))) + " ";
    CalibLog 1, "firmware level:" + STR$(INT(Revision / 16)) + "." + LTRIM$(STR$(INT(Revision AND &HF))) + " "
    setVid (dRESTORE)

END SUB

FUNCTION EndTest

    DIM rc AS INTEGER

    setVid (dPROMPT)
    ePRINT "End Calibration."
    rc = Response("<Enter=End Calib.>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")
    
    IF rc = 0 THEN
	IF Revision >= REVISION24 THEN
	    PRINT "Locking EEPROM."
	    PRINT #1, CHR$(&HFD)
	    IF RxData = 0 THEN
		CalibLog 1, "EEPROM is locked."
		PRINT "EEPROM is locked."
	    ELSE
		dspError "EEPROM lock failed."
	    END IF
	END IF

	CalibLog 1, "End of Calibration."

    END IF

    EndTest = rc

END FUNCTION

SUB ePRINT (msg AS STRING)        ' emphasized print
       
	setVid (dSAVE)
	setVid (dEPRINT)
	PRINT msg
	setVid (dRESTORE)
	LOCATE CSRLIN + 1

END SUB

FUNCTION EscCont

REM     0 => Esc hit
REM     1 => any other key hit

    DIM action AS STRING

    DO
	action = INKEY$
	IF action = CHR$(&H1B) THEN
	    EscCont = 0
	    EXIT DO
	END IF

	IF action <> "" THEN
	    EscCont = 1
	    EXIT DO
	END IF

    LOOP

END FUNCTION

SUB FlushRcvr

    WHILE RxData <> -1
    WEND

END SUB

SUB Form

    KEY OFF
    WIDTH 80, 25
    COLOR 7, 1
    CLS
    COLOR 1, 7
    LOCATE 1, 1, 1
    PRINT STRING$(80, 196);
    LOCATE 1, (80 - LEN(SIGNON)) / 2 - 1
    PRINT " " + SIGNON + " "
    PRINT
    COLOR 7, 1
    LOCATE 3, 1

PRINT "  Ŀ Ŀ"
PRINT "                Calibration Parameters                  Battery Data      "
PRINT "  Ĵ Ĵ"
PRINT "                  Original        Calibrated         Voltage:           "
PRINT "  Ĵ      Current:           "
PRINT "    Parameter   Slope  Offset  Slope  Offset   Temperature:           "
PRINT "  Ĵ     Capacity:           "
PRINT "   Voltage                                         Percent:           "
PRINT "   Current              n/a            n/a           Flags:           "
PRINT "   Temperature                                      Errors:           "
PRINT "   "

    setVid (dPROMPT)

END SUB

SUB GetEEPROMfile

    DIM i AS INTEGER
    DIM j AS INTEGER
    DIM k AS INTEGER
    DIM addr AS INTEGER
    DIM label AS STRING
    DIM equal AS STRING
    DIM Value AS STRING
    DIM position AS INTEGER
    DIM Reason AS STRING
    DIM byte AS INTEGER
    DIM length AS INTEGER
    DIM record AS STRING
    DIM chksum AS INTEGER

    SHARED Filename AS STRING
    SHARED EEdata() AS INTEGER

    DIM rc AS INTEGER

TryAgain:

    setVid dPROMPT
    PRINT "Open an EEPROM Initialization Data File."
    rc = Response("<Enter=Open>  <Tab=skip to Next>  <BackTab=skip to Previous>", "")
    
    IF rc <> 0 THEN EXIT SUB

    INPUT "Enter EEPROM DATA filename: ", Filename

    SysErr = 0
    OPEN Filename FOR INPUT AS #2

    IF SysErr = 0 THEN

	k = 0
	addr = 0
       
	FOR i = 1 TO 8
	    'input intel INHX8M record
	    INPUT #2, record
	       
	    length = LEN(record)
	    Reason = "??? Bad Record Length" + CHR$(13) + CHR$(10) + "  is: " + STR$(length) + "  should be: 16"
	    IF length <> 43 THEN GOTO BadRecord

	    Reason = "??? Record does not begin with :"
	    IF MID$(record, 1, 1) <> ":" THEN GOTO BadRecord

	    ' Check for recordlength = 16
	    Reason = "??? Byte Count not equal to 10 (hex)"
	    IF VAL("&H" + MID$(record, 2, 2)) <> &H10 THEN GOTO BadRecord

	    ' Check for address = (i-1)*&h0010
	    Reason = "??? Bad Address"
	    IF VAL("&H" + MID$(record, 4, 4)) <> addr THEN GOTO BadRecord
	    addr = addr + &H10    ' 0000 <= addr <= 0070 (hex)

	    ' Check for recordtype = 00
	    Reason = "??? Recordtype not equal to 0"
	    IF VAL("&H" + MID$(record, 8, 2)) <> 0 THEN GOTO BadRecord
	       
	    chksum = 0
	    ' convert data
	    FOR j = 0 TO 16
		byte = VAL("&H" + MID$(record, 10 + 2 * j, 2))
		chksum = chksum + byte
		IF (j < 16) THEN
		    EEdata(k) = byte
		    k = k + 1
		END IF
	       
	    NEXT j

	    Reason = "??? Bad Checksum"
	    IF (chksum AND &HFF) <> 0 THEN GOTO BadRecord
	NEXT i

	' Check for end of file record
	INPUT #2, record

	Reason = "??? Bad or missing end of file record"
	IF record <> ":00000001ff" GOTO BadRecord

	EEdataAvail = 1
	CalibLog 1, "opened EEPROM data file:" + Filename

	CLOSE #2

    ELSE
	dspError "Could NOT open file: " + Filename
	CLOSE #2
	GOTO TryAgain
    END IF

    EXIT SUB

BadRecord:
	dspError Reason + CHR$(13) + record
	CLOSE #2
	GOTO TryAgain

END SUB

SUB GetLogFile

    DIM rc AS INTEGER
    DIM LogFile AS STRING

    setVid dPROMPT
    PRINT "Open a calibration result log file."
    rc = Response("<Enter=Open>  <Tab=skip to Next>  <BackTab=skip to Previous>", "")
   
    IF rc = 0 THEN
	INPUT "Enter Result Log Filename: ", LogFile

	IF hLogFile THEN
	    CLOSE hLogFile
	    hLogFile = 0
	END IF

	SysErr = 0
	OPEN LogFile FOR APPEND AS #3
	IF SysErr = 0 THEN
	    hLogFile = 3
	    CalibLog 2, ""
	ELSE
	    CLOSE #3
	END IF

    END IF

END SUB

SUB GetPartNumber

    DIM rc AS INTEGER

    setVid dPROMPT

    IF rc = 0 THEN
	setVid dPROMPT
	INPUT "Enter Part/Serial Number: ", PartNumber
	CalibLog 0, ""
	CalibLog 1, "begin calibration on TG part " + PartNumber
    END IF

END SUB

SUB GetPortNumber

    DIM rc AS INTEGER

    setVid (dPROMPT)
    PRINT "Enter COM Port Number (1, 2, 3, or 4)."
    rc = Response("<1, 2, 3, or 4=Port Number>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")
    IF (rc > 1) THEN
	IF PortNumber THEN
	    CLOSE #1
	    PortNumber = 0
	END IF

	PortNumber = rc - &H30

	SysErr = 0
	OPEN "COM" + CHR$(rc) + ":9600,N,8,1,BIN,RS,CS2000,DS0,CD0" FOR RANDOM AS #1
	IF SysErr <> 0 THEN
	    rc = 1
	    PortNumber = 0
	ELSE
	    rc = 0
	    CalibLog 1, "opened comm port:" + STR$(PortNumber)
	END IF

    END IF

END SUB

SUB GetRefCur

    DIM rc AS INTEGER
    DIM userinput AS STRING

    setVid dPROMPT
    PRINT "Change Reference Load (" + LTRIM$(STR$(RefCur)) + " milliamps) for Current Calibration."
    rc = Response("<Enter=Change>  <Tab=skip to Next>  <BackTab=skip to Previous>", "")

    IF rc = 0 THEN
	INPUT "Enter new load in milliamps: ", userinput
	RefCur = VAL(userinput)
	CalibLog 1, "changed ref load to" + (STR$(RefCur)) + " mamps for current calibration."
    END IF

END SUB

FUNCTION Initialize

    DIM rc AS INTEGER

    setVid dPROMPT
    ePRINT "Initialize Calibration Options."
    PRINT "  1. Open calibration result log file (optional)."
    PRINT "  2. Select comm port."
    PRINT "  3. Open EEPROM data file (optional)."
    PRINT "  4. Specify load for current calibration (optional)."

    rc = Response("<Enter=Initialize>  <Tab=skip to Next>  <BackTab=skip to Previous>", "")
    IF rc = 0 THEN
       
	setVid dPROMPT
	GetLogFile
	GetPortNumber
	GetEEPROMfile
	GetRefCur

    END IF

    Initialize = rc

END FUNCTION

FUNCTION More

    setVid (dPROMPT)
    ePRINT "Exit Calibration Program"
    More = Response("<Enter=Exit>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")

END FUNCTION

SUB PrintReport

    DIM lin AS INTEGER
    DIM Col AS INTEGER
    DIM msg AS STRING

    PRINT "Reading Battery Data.";

    setVid (dSAVE)

    setVid (dBATTERY)
    lin = CSRLIN
    Col = POS(0)

    IF ReadBatteryData = 1 THEN
	IF FlagByte AND &H1 THEN Current = -Current

	LOCATE lin, Col
	PRINT USING "##.## v"; Voltage / 1000!
	LOCATE lin + 1, Col
	PRINT USING "#### ma"; Current
	LOCATE lin + 2, Col
	PRINT USING "###.# C"; Temperature
	LOCATE lin + 3, Col
	PRINT USING "#### maH"; Capacity
	LOCATE lin + 4, Col
	PRINT USING "## %"; PctChg
	LOCATE lin + 5, Col
	PRINT HEX$(ErrorByte)
	LOCATE lin + 6, Col
	PRINT HEX$(FlagByte)

	msg = "battery data:" + STR$(Voltage / 1000!) + "," + STR$(Current / 1000!) + ","
	msg = msg + STR$(Temperature / 256!) + "," + STR$(Capacity / 1000!) + "," + STR$(PctChg) + "," + STR$(ErrorByte) + "," + STR$(FlagByte)
	CalibLog 1, msg
    ELSE
	dspError "Error Reading Battery Data"
    END IF

    setVid (dRESTORE)

END SUB

SUB prtFactors

    CalibLog 1, "original voltage slope:" + STR$((OrigParams(1) * 256 + OrigParams(0)) / 1000!)
    CalibLog 1, "original voltage offset:" + STR$(OrigParams(3) * 256 + OrigParams(2))
    CalibLog 1, "original temperature slope:" + STR$((OrigParams(7) * 256 + OrigParams(6)) / 256!)
    CalibLog 1, "original temperature offset:" + STR$(OrigParams(9) * 256 + OrigParams(8))
    CalibLog 1, "original current slope:" + STR$(((OrigParams(5) * 256 + OrigParams(4)) / 256!))

END SUB

FUNCTION ReadBatteryData
    
DIM lsb AS INTEGER
DIM msb AS INTEGER
REM: Read Battery Data

    IF PortNumber = 0 THEN EXIT FUNCTION

    PRINT #1, CHR$(&HF4);

    DO
	lsb = RxData
	IF lsb = -1 THEN EXIT DO
	msb = RxData
	IF msb = -1 THEN EXIT DO
	Voltage = CLNG(msb) * 256 + lsb

	lsb = RxData                    ' signed temperature
	IF lsb = -1 THEN EXIT DO
	msb = RxData
	IF msb = -1 THEN EXIT DO
	Temperature = CLNG(lsb) / 256 + msb
	IF Temperature >= &H8000& THEN Temperature = -(&H10000 - Temperature)

	lsb = RxData
	IF lsb = -1 THEN EXIT DO
	msb = RxData
	IF msb = -1 THEN EXIT DO
	Current = CLNG(msb) * 256 + lsb

	lsb = RxData
	IF lsb = -1 THEN EXIT DO
	PctChg = lsb

	lsb = RxData
	IF lsb = -1 THEN EXIT DO
	ErrorByte = lsb

	lsb = RxData
	IF lsb = -1 THEN EXIT DO
	msb = RxData
	IF msb = -1 THEN EXIT DO
	Capacity = CLNG(msb) * 256 + lsb

	IF RxData = -1 THEN EXIT DO
	IF RxData = -1 THEN EXIT DO

	lsb = RxData
	IF lsb = -1 THEN EXIT DO
	FlagByte = lsb

	IF RxData = -1 THEN EXIT DO
	IF RxData = -1 THEN EXIT DO
	IF RxData = -1 THEN EXIT DO

	ReadBatteryData = 1
	EXIT FUNCTION

    LOOP
    ReadBatteryData = 0

END FUNCTION

FUNCTION ReadEE (location AS INTEGER)

    PRINT #1, CHR$(&HF0) + CHR$(location);
    ReadEE = RxData

END FUNCTION

FUNCTION ReadRam (location AS INTEGER)

    DIM Value AS INTEGER

    PRINT #1, CHR$(&HF5) + CHR$(location);
    Value = RxData
    ReadRam = Value

END FUNCTION

FUNCTION ResetTG

    PRINT "Resetting TrueGauge.  One moment, please."
    CalibLog 1, "resetting TrueGauge"
    PRINT #1, CHR$(&HFB);
    IF RxData = &HFB THEN
	PRINT "Reset successful"
	ResetTG = 1
	UnlockEEPROM
    ELSE
	dspError "Reset Failed"
	BEEP
	ResetTG = 0
    END IF

END FUNCTION

FUNCTION Response (msg AS STRING, Valid AS STRING)
    
    DIM l1 AS INTEGER

    IF msg <> "" THEN
	setVid (dSAVE)
	setVid (dHELP)
	PRINT msg;
	setVid (dRESTORE)
    END IF

    DO
	X$ = INKEY$

	l1 = LEN(X$)

	IF l1 = 1 THEN
	    IF ASC(X$) = 9 THEN      ' process tab key
		rc = 1
		EXIT DO
	    END IF

	    IF ASC(X$) = 13 THEN     ' process enter key
		rc = 0
		EXIT DO
	    END IF

	    IF INSTR(Valid, X$) THEN
		rc = ASC(X$)
		EXIT DO
	    END IF

	END IF

	IF l1 = 2 THEN
		IF ASC(MID$(X$, 2, 1)) = 15 THEN        ' process backtab key
		    rc = -1
		    EXIT DO
		END IF
	END IF

	IF l1 THEN BEEP

    LOOP

    Response = rc

    IF msg <> "" THEN clrLine25          ' clear 25th line

END FUNCTION

FUNCTION RxData

    DIM start AS SINGLE
    DIM Value AS INTEGER

    start = TIMER + 2

    DO
	IF PortNumber = 0 THEN
	    RxData = -1
	    EXIT FUNCTION
	END IF

	IF LOC(1) <> 0 THEN
	    Value = ASC(INPUT$(1, #1))
	    EXIT DO
	END IF

	IF TIMER > start THEN
		Value = -1
		EXIT DO
	END IF
    LOOP

    RxData = Value

END FUNCTION

FUNCTION SetNormalization

	DIM rc AS INTEGER

	setVid (dPROMPT)
	ePRINT "Set Normalization Point."
	rc = Response("<Enter=Set NP>  <Tab=skip to Next>  <BackTab=skip to Previous>", "1234")

	IF rc = 0 THEN
	    UnlockEEPROM
	    PRINT "Setting Normalization Point. One moment, please."
	    rc = ResetTG
	    PRINT #1, CHR$(&HFC);
	    CalibLog 1, "setting Normalization Point."
	    IF RxData = &HFC THEN
		PRINT "Normalization point set"
		rc = 0
	    ELSE
		dspError "Set Normalization Point Failed"
		BEEP
		rc = 1
	    END IF
	END IF

    SetNormalization = rc

END FUNCTION

SUB setVid (dType AS INTEGER) STATIC

CONST lineBOX = 3
CONST lineVERSION = 14
CONST linePROMPT = 17
CONST lineHELP = 25
CONST lineERROR = 25
CONST lineBATTERY = lineBOX + 3
CONST lineVOLT = lineBOX + 7
CONST lineCURRENT = lineBOX + 8
CONST lineTEMP = lineBOX + 9
CONST lineMENUBAR = 16

CONST colVERSION = 55
CONST colSLOPEORIG = 18
CONST colOFFSETORIG = 27
CONST colSLOPE = 36
CONST colOFFSET = 44
CONST colBATTERY = 68
CONST colCOMMPORT = 10

CONST fgdPROMPT = 0         ' black
CONST bgdPROMPT = 7         ' white
CONST fgdEPRINT = 4         ' red
CONST bgdEPRINT = 7         ' white
CONST fgdERROR = 7          ' white
CONST bgdERROR = 4          ' red
CONST fgdPARMS = 15         ' bright white
CONST bgdPARMS = 1          ' blue
CONST bgdMENUBAR = 3        ' cyan
CONST fgdMENUBAR = 0        ' black
CONST bgdMENUITEM = 3       ' cyan
CONST fgdMENUITEM = 0       ' black
CONST fgdSELMENUITEM = 4    ' red
CONST bgdSELMENUITEM = 3    ' cyan
CONST fgdHELP = 0           ' black
CONST bgdHELP = 3           ' cyan

CONST ITEMWIDTH = 9

DIM stColor(8) AS INTEGER  ' save / restore video state variables
DIM stLine(8)  AS INTEGER
DIM stCol(8)   AS INTEGER
STATIC stSP AS INTEGER

DIM MenuItems(NBRMENUITEMS) AS STRING

DIM i AS INTEGER
STATIC curLine AS INTEGER
STATIC curCol AS INTEGER

    MenuItems(0) = "   Setup"
    MenuItems(1) = "  Begin "
    MenuItems(2) = " Set NP "
    MenuItems(3) = " Voltage"
    MenuItems(4) = " Current"
    MenuItems(5) = "  Temp  "
    MenuItems(6) = "  End   "
    MenuItems(7) = " Restore"
    MenuItems(8) = " Quit   "

    SELECT CASE dType
	CASE dPROMPT
	    COLOR fgdPROMPT, bgdPROMPT

	    FOR i = linePROMPT TO 25
		LOCATE i, 1
		PRINT STRING$(80, 32);
	    NEXT
	    LOCATE linePROMPT + 1, 1

	CASE dEPRINT
	    COLOR fgdEPRINT, bgdEPRINT

	CASE dERROR
	    LOCATE lineERROR, 1
	    COLOR fgdERROR, bgdERROR

	CASE dVERSION
	    LOCATE lineVERSION, colVERSION
	    COLOR fgdERROR, bgdERROR

	CASE dSAVE
		stSP = stSP + 1
		stCol(stSP) = POS(0)
		stLine(stSP) = CSRLIN
		stColor(stSP) = SCREEN(stLine(stSP), stCol(stSP), 1)

	CASE dRESTORE
		LOCATE stLine(stSP), stCol(stSP)
		COLOR stColor(stSP) AND &HF, INT(stColor(stSP) / 16)
		stSP = stSP - 1

	CASE dBATTERY
		LOCATE lineBATTERY, colBATTERY
		COLOR fgdPARMS, bgdPARMS

	CASE dCURSLOPE
		LOCATE lineCURRENT, colSLOPE
		COLOR fgdPARMS, bgdPARMS

	CASE dTEMPSLOPE
		LOCATE lineTEMP, colSLOPE
		COLOR fgdPARMS, bgdPARMS

	CASE dTEMPOFF
		LOCATE lineTEMP, colOFFSET
		COLOR fgdPARMS, bgdPARMS

	CASE dVOLTSLOPE
		LOCATE lineVOLT, colSLOPE
		COLOR fgdPARMS, bgdPARMS

	CASE dVOLTOFF
		LOCATE lineVOLT, colOFFSET
		COLOR fgdPARMS, bgdPARMS

	CASE dCURSLOPEORIG
		LOCATE lineCURRENT, colSLOPEORIG
		COLOR fgdPARMS, bgdPARMS

	CASE dTEMPSLOPEORIG
		LOCATE lineTEMP, colSLOPEORIG
		COLOR fgdPARMS, bgdPARMS

	CASE dTEMPOFFORIG
		LOCATE lineTEMP, colOFFSETORIG
		COLOR fgdPARMS, bgdPARMS

	CASE dVOLTSLOPEORIG
		LOCATE lineVOLT, colSLOPEORIG
		COLOR fgdPARMS, bgdPARMS

	CASE dVOLTOFFORIG
		LOCATE lineVOLT, colOFFSETORIG
		COLOR fgdPARMS, bgdPARMS

	CASE dMENUBAR
	    LOCATE lineMENUBAR, 1
	    COLOR fgdMENUBAR, bgdMENUBAR
	    PRINT STRING$(80, 32)

	    FOR i = 0 TO NBRMENUITEMS - 1
		IF i = MenuIndex THEN
		    COLOR fgdSELMENUITEM, bgdSELMENUITEM
		ELSE
		    COLOR fgdMENUITEM, bgdMENUITEM
		END IF

	    LOCATE lineMENUBAR, i * ITEMWIDTH + 1
		PRINT MenuItems(i)
	    NEXT

	CASE dHELP
	    LOCATE lineHELP, 1
	    COLOR fgdHELP, bgdHELP
	    PRINT STRING$(80, 32);
	    LOCATE lineHELP, 1

	CASE dCOMMPORT
	    LOCATE lineVERSION, colCOMMPORT
	    COLOR fgdERROR, bgdERROR

    END SELECT

END SUB

FUNCTION Test

    DIM rc AS INTEGER
    DIM tmp AS INTEGER

    setVid dPROMPT
    ePRINT "Start Calibration Sequence."
    PRINT "  1. Specify part number."
    PRINT "  2. Verify broadcast mode is off."
    PRINT "  3. Verify comm link to TrueGauge is operational."
    PRINT "  4. Load current calibration factors."
    IF EEdataAvail THEN PRINT "  5. Initialize EEPROM."

    rc = Response("<Enter=Start>  <Tab=skip to Next>  <BackTab=skip to Previous>", "")
  
    IF rc = 0 THEN
	GetPartNumber                ' get TG part/serial number
	tmp = BroadCastOff           ' Check if TG is in broadcast mode
	IF tmp = 0 THEN
	    tmp = CommCheck          '   no, check comm link
	ELSE                         '   yes, and do not disable
	    PortNumber = 0
	    tmp = 1
	END IF

	IF tmp = 0 THEN
	    IF EEdataAvail THEN DnLoadEEPROM        ' download eedata to TrueGauge
	END IF

    END IF

    Test = rc

END FUNCTION

SUB UnlockEEPROM

    DIM Value AS INTEGER

    IF Revision >= REVISION24 THEN
	IF ReadRam(&H1C) <> &HAA THEN
	    PRINT "Unlocking EEPROM. ";
	    PRINT #1, CHR$(&HFD)    'Toggle EEPROM Lock
	    Value = RxData
	    IF Value = &HAA THEN
		PRINT "EEPROM unlocked."
		CalibLog 1, "EEPROM unlocked."
	    ELSE
		dspError "EEPROM unlock Failed"
	    END IF
	END IF
    END IF

END SUB

SUB UpLoadOrig

    DIM i AS INTEGER
    DIM byte AS INTEGER
				  
    PRINT "Uploading original calibration factors: []";
    LOCATE , POS(0) - 11

    FOR i = 0 TO 9
	IF PortNumber = 0 THEN EXIT SUB
	byte = ReadEE(50 + i)
	IF byte = -1 THEN
	    dspError "Error reading EEPROM location" + STR$(50 + i)
	    EXIT SUB
	END IF

	PRINT "*";
	OrigParams(i) = byte

    NEXT

    dspFactors
    prtFactors

END SUB

FUNCTION WriteEE (location AS INTEGER, Value AS INTEGER)

    DIM rc AS INTEGER

    PRINT #1, CHR$(&HF1) + CHR$(location) + CHR$(Value);
    rc = RxData
    WriteEE = rc

END FUNCTION

FUNCTION YesNo

    DO
	X$ = INKEY$

	IF X$ = "Y" OR X$ = "y" THEN
		YesNo = 1
		EXIT DO
	END IF


	IF X$ = "N" OR X$ = "n" THEN
		YesNo = 0
		EXIT DO

	END IF

	IF X$ <> "" THEN BEEP

    LOOP
   
    PRINT " " + X$

END FUNCTION

